﻿using System.Linq;

using VSTalk.Engine.Core.EntitySession;
using VSTalk.Engine.Core.Feed;
using VSTalk.Model;

using agsXMPP;

namespace VSTalk.Engine.Core.ChatSession
{
    public class InterlocutorChatSession : IInterlocutorChatSession, IMessageConsumer
    {
        public Jid ActiveResource { get; private set; }

        public Interlocutor Interlocutor
        {
            get { return _interlocutor; }
        }

        private readonly Interlocutor _interlocutor;

        public static IMessageFeed MessageFeed
        {
            get { return AppServices.Get<IMessageFeed>(); }
        }

        private static ISessionModel SessionModel
        {
            get { return AppServices.Get<ISessionModel>(); }
        }

        public IInterlocutorContext InterlocutorContext
        {
            get { return SessionModel.GetInterlocutorContext(_interlocutor); }
        }

        public InterlocutorChatSession(Interlocutor interlocutor)
        {
            _interlocutor = interlocutor;
            ActiveResource = InitResource();
            ReadHistory();
        }

        private Jid InitResource()
        {
            if (InterlocutorContext.Resources.All(resource => resource.State == ContactState.Offline))
            {
                return InterlocutorContext.Bare;
            }
            if (InterlocutorContext.Resources.Count(resource => resource.State == ContactState.Online) == 1)
            {
                return InterlocutorContext.Resources.Single(resource => resource.State == ContactState.Online).Jid;
            }
            return InterlocutorContext.Bare;
        }

        private void ReadHistory()
        {
            foreach (var msg in MessageFeed.GetMessages(Predicate))
            {
                Consume(msg);
            }
        }

        public void Consume(IMessage message)
        {
            ActiveResource = GetActiveResource(message);
        }

        private Jid GetActiveResource(IMessage message)
        {
            var from = message.XmppMessage.From;
            var resource = InterlocutorContext.Resources.FirstOrDefault(interlocutorResource => Equals(interlocutorResource.Jid, @from));
            if (resource != null && resource.State == ContactState.Online)
            {
                return resource.Jid;
            }
            return InitResource();
        }

        public bool Predicate(IMessage message)
        {
            return AddressPatterns.From(_interlocutor)(message);
        }
    }
}