﻿using System;

using VSTalk.Engine.Core.Context;
using VSTalk.Engine.Core.Event.Events;
using VSTalk.Model;
using VSTalk.Tools;

namespace VSTalk.Engine.Core.ClientManager
{
    public class ClientManager : IClientManager
    {
        private static IEventAggregator EventBus
        {
            get { return AppServices.Get<IEventAggregator>(); }
        }

        private Account Account
        {
            get { return AppServices.Get<IModelContext>().Account; }
        }

        public void AddClient(IClientInfo clientInfo)
        {
            var client = InitClient(clientInfo);
            Account.XmppClients.Add(client);
            EventBus.Publish(new ClientAddedEvent
            {
                ClientId = client.Id
            });
        }

        private static Client InitClient(IClientInfo clientInfo)
        {
            var client = new Client();
            client.Id = Guid.NewGuid();
            client.Login = clientInfo.Login;
            client.Server = clientInfo.Server;
            client.Password = clientInfo.Password;
            return client;
        }

        public void RemoveClient(Client client)
        {
            EventBus.Publish(new ClientRemovingEvent
            {
                ClientId = client.Id
            });
            Account.XmppClients.Remove(client);
        }
    }
}