﻿using System;
using System.Collections.Generic;

using VSTalk.Logger;

using agsXMPP;

using log4net;

namespace VSTalk.Engine.Core.Disco
{
    public class DiscoService : IDiscoService, IDisposable
    {
        private static readonly ILog _log = LoggerFactory.Create<DiscoRepository>();

        private readonly DiscoRepository _discoRepository = new DiscoRepository();

        private readonly IList<IDiscoGrabber> _grabbers = new List<IDiscoGrabber>();

        public DiscoService()
        {
            _log.Info("Initializing " + typeof(DiscoService).FullName);

            var interlocutorGrabber = new InterlocutorDiscoGrabber();
            interlocutorGrabber.Initialize(_discoRepository);
            interlocutorGrabber.StartListen();

            //var serverGrabber = new ServerDiscoGrabber();
            //serverGrabber.Initialize(_discoRepository);
            //_grabbers.Add(serverGrabber);

            _grabbers.Add(interlocutorGrabber);
            
            _log.Info("Initialized " + typeof(DiscoService).FullName);
        }

        public bool HasFeature(Jid jid, string uri)
        {
            var disco = _discoRepository.GetDisco(jid);
            if (disco == null)
            {
                return false;
            }
            return disco.HasFeature(uri);
        }

        public void Dispose()
        {
            foreach (var discoGrabber in _grabbers)
            {
                discoGrabber.StopListen();
            }
        }
    }
}