﻿using System.Collections.Generic;
using System.Windows.Threading;

using VSTalk.Engine.Core.Context;
using VSTalk.Engine.Core.EntitySession;
using VSTalk.Engine.Core.XmppConnection;
using VSTalk.Model;
using VSTalk.Tools;

using agsXMPP;
using agsXMPP.protocol.client;
using agsXMPP.protocol.iq.disco;

namespace VSTalk.Engine.Core.Disco
{
    public class InterlocutorDiscoGrabber : IDiscoGrabber
    {
        private DiscoRepository _repository;
        private IListenerUnit _modelContextUnit;

        public IModelContext ModelContext
        {
            get { return AppServices.Get<IModelContext>(); }
        }

        public ISessionModel SessionModel
        {
            get { return AppServices.Get<ISessionModel>(); }
        }

        public void Initialize(DiscoRepository repository)
        {
            _repository = repository;
        }

        public void StartListen()
        {
            _modelContextUnit = ModelContext.Interlocutors.HandleCollectionChanges(initialOrAdd:SubscribeInterlocutor);
        }

        public void StopListen()
        {
            _modelContextUnit.Stop();
        }

        private void SubscribeInterlocutor(Interlocutor obj, IPrologue p)
        {
            var listener = new InterlocutorListener();
            var client = ModelContext.GetClientByContact(obj);

            listener.ClientContext = SessionModel.GetClientContext(client);
            listener.InterlocutorContext = SessionModel.GetInterlocutorContext(obj);
            listener.DiscoGrabber = this;

            p.Add(listener.InterlocutorContext.Resources.HandleCollectionChanges(initialOrAdd:listener.SubscribeResource));
        }

        private class InterlocutorListener
        {
            private readonly Dispatcher _dispatcher;
            private readonly IList<Jid> _pendingIq = new List<Jid>();

            public IClientContext ClientContext { get; set; }
            public IInterlocutorContext InterlocutorContext { get; set; }
            public InterlocutorDiscoGrabber DiscoGrabber { get; set; }

            private IClientConnection ClientConnection
            {
                get { return AppServices.Get<IConnectionManager>().GetOrCreateConnection(ClientContext.Client); }
            }

            public InterlocutorListener()
            {
                _dispatcher = Dispatcher.CurrentDispatcher;
            }

            public void SubscribeResource(IInterlocutorResource obj, IPrologue p)
            {
                if (obj.State == ContactState.Online)
                {
                    ResourceOnline(obj);
                }
                p.Add(obj.SubscribeToChange(() => obj.State, sender => sender.State == ContactState.Online, ResourceOnline));
            }

            private void ResourceOnline(IInterlocutorResource sender)
            {
                if (DiscoGrabber._repository.ContainsJid(sender.Jid) || _pendingIq.Contains(sender.Jid))
                {
                    return;
                }
                _pendingIq.Add(sender.Jid);
                var discoManager = new DiscoManager(ClientConnection.Connection);
                discoManager.AutoAnswerDiscoInfoRequests = false;
                discoManager.DiscoverInformation(sender.Jid, _dispatcher.Wrap<object, IQ, object>(ProcessDiscoInfo).Exec);
            }

            private void ProcessDiscoInfo(object sender, IQ iq, object data)
            {
                _pendingIq.Remove(iq.From);
                var disco = iq.Query as DiscoInfo;
                if (disco == null)
                {
                    return;
                }
                DiscoGrabber._repository.UpdateDisco(iq.From, disco);
            }
        }
    }
}