﻿using System.Collections.Generic;
using System.Windows.Threading;

using agsXMPP;
using agsXMPP.protocol.client;
using agsXMPP.protocol.iq.disco;

using VSTalk.Engine.Core.Context;
using VSTalk.Engine.Core.EntitySession;
using VSTalk.Engine.Core.XmppConnection;
using VSTalk.Model;
using VSTalk.Tools;

namespace VSTalk.Engine.Core.Disco
{
    public class ServerDiscoGrabber : IDiscoGrabber
    {
        private DiscoRepository _repository;

        private Dispatcher _dispatcher;

        private readonly IList<Jid> _pendingIq = new List<Jid>();
        
        private IListenerUnit _modelContextUnit;

        public IModelContext ModelContext
        {
            get { return AppServices.Get<IModelContext>(); }
        }

        public void Initialize(DiscoRepository repository)
        {
            _repository = repository;
            _dispatcher = Dispatcher.CurrentDispatcher;
        }


        public void StartListen()
        {
            _modelContextUnit = ModelContext.Clients.HandleCollectionChanges(HandleClientChanges);
        }

        public void StopListen()
        {
            _modelContextUnit.Stop();
        }

        private void HandleClientChanges(Client client, IPrologue p)
        {
            var context = AppServices.Get<ISessionModel>().GetClientContext(client);
            if (context.State == ClientState.Connected)
            {
                ExploreServer(client);
            }
            p.Add(context.SubscribeToChange(() => context.State,
                    sender => sender.State == ClientState.Connected,
                    sender => ExploreServer(client)));
        }

        private void ExploreServer(Client client)
        {
            var serverJid = new Jid(null, client.Server, null);
            if (_repository.ContainsJid(serverJid) || _pendingIq.Contains(serverJid))
            {
                return;
            }
            _pendingIq.Add(serverJid);
            var connection = AppServices.Get<IConnectionManager>().GetOrCreateConnection(client);
            var discoManager = new DiscoManager(connection.Connection);
            discoManager.AutoAnswerDiscoInfoRequests = false;
            discoManager.DiscoverInformation(serverJid, _dispatcher.Wrap<object, IQ, object>(ProcessDiscoInfo).Exec);
        }

        private void ProcessDiscoInfo(object arg1, IQ iq, object arg3)
        {
            _pendingIq.Remove(iq.From);
            var disco = iq.Query as DiscoInfo;
            if (disco == null)
            {
                return;
            }
            _repository.UpdateDisco(iq.From, disco);
        }
    }
}