﻿using System;
using System.ComponentModel;
using System.Drawing;
using System.Drawing.Imaging;
using System.IO;
using System.Text;
using VSTalk.Tools;
using agsXMPP;


namespace VSTalk.Engine.Core.EntitySession
{
    public class InterlocutorAvatar: IInterlocutorAvatar 
    {
        private string _hash;

        private Image _avatar;

        private const string STORAGE_PATH = @"\VSTalk\Avatars\";

        
        public InterlocutorAvatar(Jid interlocutorBareJid, ImageFormat imageExtension)
        {
            var path = Environment.GetFolderPath(Environment.SpecialFolder.LocalApplicationData);
            Directory.CreateDirectory(path + STORAGE_PATH);
            ImageExtension = imageExtension;
            AvatarFileName = GenerateFileName(interlocutorBareJid);
        }

        private string ConvertToASCII(string unicodeString)
        {
            var bytes = Encoding.Unicode.GetBytes(unicodeString);
            return Encoding.ASCII.GetString(bytes);
        }

        private string GenerateFileName(string interlocutorJid)
        {
            var path = Environment.GetFolderPath(Environment.SpecialFolder.LocalApplicationData);
            var file = interlocutorJid.Replace("@","_").Replace('.','_');
            var fullName = String.Format("{0}{1}{2}.{3}",path, STORAGE_PATH, file, ImageExtension.ToString().ToLower());
            return fullName;
        }

        public string AvatarFileName { get; private set; }

        public string Hash
        {
            get
            {
                return _hash;
            }
            set
            {
                if (_hash != value)
                {
                    _hash = value;
                    PropertyChanged.Notify(() => Hash);
                }
            }
        }

        public Image Avatar
        {
            get
            {
                return _avatar;
            }
            set
            {
                _avatar = value;
                PropertyChanged.Notify(() => Avatar);
            }
        }

        public ImageFormat ImageExtension { get; set; }

        public event PropertyChangedEventHandler PropertyChanged;

        public void StartListen(){}

        public void StopListen(){}
    }
}
