using System.Collections.Generic;
using System.ComponentModel;
using System.Linq;

using VSTalk.Engine.Core.Context;
using VSTalk.Engine.Core.XmppUtils;
using VSTalk.Tools;

namespace VSTalk.Engine.Core.Feed.MessageState
{
    public class AggregatedMessageCounter : IMessageConsumer, IUnreadCounter
    {
        private readonly IList<IMessage> _unreadMessages = new List<IMessage>();

        private readonly IList<IMessage> _waitingMessages = new List<IMessage>();

        private static IModelContext ModelContext
        {
            get { return AppServices.Get<IModelContext>(); }
        }

        public int Count
        {
            get { return _unreadMessages.Count; }
        }

        public bool HasMessages
        {
            get { return Count > 0; }
        }

        public void MarkAsRead(IMessage message)
        {
            if (!_unreadMessages.Contains(message))
            {
                if (Predicate(message) && !_waitingMessages.Contains(message))
                {
                    _waitingMessages.Add(message);
                }
                return;
            }
            _unreadMessages.Remove(message);
            PropertyChanged.Notify(() => Count);
            PropertyChanged.Notify(() => HasMessages);
        }

        public void Consume(IMessage message)
        {
            if (_waitingMessages.Contains(message))
            {
                _waitingMessages.Remove(message);
                return;
            }
            _unreadMessages.Add(message);
            PropertyChanged.Notify(() => Count);
            PropertyChanged.Notify(() => HasMessages);
        }

        public bool Predicate(IMessage message)
        {
            return MessageUtil.IsContentMessage(message.XmppMessage) &&
                   ModelContext.Interlocutors.Any(from => AddressPatterns.From(from)(message));
        }

        public event PropertyChangedEventHandler PropertyChanged;
    }
}