﻿using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Runtime.Serialization;
using System.Xml;

using VSTalk.Engine.Core.Feed;
using VSTalk.Logger;

using agsXMPP;
using agsXMPP.Xml.Dom;
using agsXMPP.protocol.client;

using log4net;

namespace VSTalk.Engine.Core.History
{
    public class HistoryService : IHistoryService
    {
        private static readonly string _settingFolder = Environment.GetFolderPath(Environment.SpecialFolder.LocalApplicationData);

        private static readonly string _filePath = Path.Combine(_settingFolder, HISTORY_FILE_PATH);

        private static readonly ILog _log = LoggerFactory.Create(typeof(HistoryService));

        //todo: push to config
        private const string HISTORY_FILE_PATH = @"VSTalk\vstalk_history.xml";

        public void Save()
        {
            _log.Info("Saving account history");

            var feed = AppServices.Get<IMessageFeed>();
            var messages = Read()
                    .Concat(feed.GetMessages(message => true)
                            .Select(ToStoredItem))
                    .ToList();

            if (!File.Exists(_filePath))
            {
                File.Create(_filePath).Dispose();
            }
            var writer = new FileStream(_filePath, FileMode.Create);
            var ser = new DataContractSerializer(messages.GetType());
            try
            {
                ser.WriteObject(writer, messages);
                _log.Info("Account history is saved");
            }
            catch (Exception e)
            {
                _log.Error("Cannot serialize contact list", e);
            }
            finally
            {
                writer.Close();
            }
        }

        private IEnumerable<StoredMessage> Read()
        {
            _log.Info("Loading history");
            var historyItems = new List<StoredMessage>();
            if (!File.Exists(_filePath))
            {
                return historyItems;
            }
            var fs = new FileStream(_filePath, FileMode.Open);
            try
            {
                var reader = XmlDictionaryReader.CreateTextReader(fs, new XmlDictionaryReaderQuotas());
                var ser = new DataContractSerializer(historyItems.GetType());

                // Deserialize the data and read it from the instance.
                historyItems = (List<StoredMessage>)ser.ReadObject(reader, true);

                reader.Close();
                _log.Info("History is loaded");
            }
            catch (Exception e)
            {
                _log.Error("Cannot deserialize history", e);
            }
            finally
            {
                fs.Close();
            }

            return historyItems;
        }

        public IEnumerable<IMessage> Read(Guid clientId, Jid with)
        {
            return Read()
                .Select(ToHistoryItem)
                .Where(item => AddressPatterns.With(clientId, with)(item));
        }

        private StoredMessage ToStoredItem(IMessage arg)
        {
            return new StoredMessage
            {
                Date = arg.Date,
                MessageXml = arg.XmppMessage.ToString(),
                ClientId = arg.ClientId,
            };
        }

        private HistoryItem ToHistoryItem(StoredMessage arg)
        {
            return new HistoryItem
            {
                Date = arg.Date,
                XmppMessage = ToMessage(arg.MessageXml),
                ClientId = arg.ClientId
            };
        }

        private Message ToMessage(string messageXml)
        {
            Document doc = new Document();
            doc.LoadXml(messageXml);
            return (Message)doc.RootElement;
        }
    }
}