using System;
using System.Collections.Generic;
using System.Linq;

using VSTalk.Engine.Core.Context;
using VSTalk.Engine.Core.XmppUtils;
using VSTalk.Model;

using agsXMPP;

namespace VSTalk.Engine.Core.XmppConnection.RosterObserver
{
    public class RosterListener : IRosterListener
    {
        private readonly Client _client;

        public RosterListener(Client client)
        {
            _client = client;
        }

        public void CreateInterlocutor(IRosterInfo info)
        {
            var name = string.IsNullOrEmpty(info.Name)
                ? info.Jid.User
                : info.Name;
            if (string.IsNullOrEmpty(name)) // also posible
            {
                name = info.Jid.Server;
            }
            _client.Contacts.Add(new Interlocutor
            {
                Id = Guid.NewGuid(),
                Name = name,
                Server = info.Jid.Server,
                User = info.Jid.User,
                Subscribed = info.Subscribed,
            });
        }

        public void UpdateInterlocutor(IRosterInfo info)
        {
            var targetContact = AppServices.Get<IModelContext>().GetContactByJid(_client.Id, info.Jid);

            targetContact.Name = string.IsNullOrEmpty(info.Name)
                    ? targetContact.Name
                    : info.Name;
            if (string.IsNullOrEmpty(targetContact.Name))
            {
                targetContact.Name = info.Jid.User;
            }
            if (string.IsNullOrEmpty(targetContact.Name)) // also posible
            {
                targetContact.Name = info.Jid.Server;
            }
            targetContact.User = info.Jid.User;
            targetContact.Server = info.Jid.Server;
            targetContact.Subscribed = info.Subscribed;
        }

        public void RemoveInterlocutor(Jid id)
        {
            var targetContact = AppServices.Get<IModelContext>().GetContactByJid(_client.Id, id);
            _client.Contacts.Remove(targetContact);
        }

        public void FilterInterlocutors(IList<Jid> recivedIds)
        {
            var toDelete = _client.Contacts
                    .Where(interlocutor => recivedIds.All(jid => !string.Equals(jid.Bare, XmppIdConverter.Jid(interlocutor), StringComparison.InvariantCultureIgnoreCase)))
                    .ToList();
            foreach (var interlocutor in toDelete)
            {
                _client.Contacts.Remove(interlocutor);
            }
        }
    }
}