﻿using System.Collections.Generic;
using System.Diagnostics;
using System.Windows.Threading;

using VSTalk.Tools;

using agsXMPP;
using agsXMPP.protocol.iq.roster;

namespace VSTalk.Engine.Core.XmppConnection.RosterObserver
{
    public class RosterObesrver : IListener
    {
        private readonly IRosterListener _listener;
        private readonly IRosterContainer _container;
        private readonly XmppClientConnection _connection;

        private readonly Dispatcher _dispatcher;

        private IList<Jid> _initialIds;
        private IActionWrapper<object> _onRosterStart;
        private IActionWrapper<object, RosterItem> _onRosterItem;
        private IActionWrapper<object> _onRosterEnd;

        public RosterObesrver(XmppClientConnection connection, IRosterListener listener, IRosterContainer container)
        {
            _dispatcher = Dispatcher.CurrentDispatcher;

            _connection = connection;
            _listener = listener;
            _container = container;
        }


        public void StartListen()
        {
            _onRosterStart = _dispatcher.Wrap<object>(OnRosterStart);
            _connection.OnRosterStart += _onRosterStart.Exec;

            _onRosterItem = _dispatcher.Wrap<object, RosterItem>(OnRosterItem);
            _connection.OnRosterItem += _onRosterItem.Exec;

            _onRosterEnd = _dispatcher.Wrap<object>(OnRosterEnd);
            _connection.OnRosterEnd += _onRosterEnd.Exec;
        }

        public void StopListen()
        {
            _connection.OnRosterStart -= _onRosterStart.Exec;
            _connection.OnRosterItem -= _onRosterItem.Exec;
            _connection.OnRosterEnd -= _onRosterEnd.Exec;
        }

        private void SetHandlers()
        {
            }

        private void OnRosterStart(object sender)
        {
            Debug.WriteLine("Reciving Roster " + _connection.MyJID);
            _initialIds = new List<Jid>();
        }

        private void OnRosterItem(object sender, RosterItem item)
        {
            if (_initialIds != null)
            {
                _initialIds.Add(item.Jid);
            }

            if (_container.Contains(item.Jid) && item.Subscription == SubscriptionType.remove)
            {
                _listener.RemoveInterlocutor(item.Jid);
            }
            else if (_container.Contains(item.Jid) && item.Subscription != SubscriptionType.remove)
            {
                _listener.UpdateInterlocutor(RosterInfo.Create(item));
            }
            else if (!_container.Contains(item.Jid) && item.Subscription != SubscriptionType.remove)
            {
                _listener.CreateInterlocutor(RosterInfo.Create(item));
            }
        }

        private void OnRosterEnd(object sender)
        {
            _listener.FilterInterlocutors(_initialIds);
            _initialIds = null;
            Debug.WriteLine("Roster recived " + _connection.MyJID);
        }

        private class RosterInfo : IRosterInfo
        {
            public Jid Jid { get; private set; }

            public string Name { get; private set; }

            public bool Subscribed { get; private set; }

            public static RosterInfo Create(RosterItem item)
            {
                bool isSubscribed = item.Subscription == SubscriptionType.both ||
                                    item.Subscription == SubscriptionType.to;
                var name = item.Name;
                return new RosterInfo
                {
                    Jid = item.Jid,
                    Name = name,
                    Subscribed = isSubscribed
                };
            }
        }
    }
}