﻿using System.Collections.ObjectModel;
using System.Linq;

using System.Windows.Controls;

using VSTalk.Engine.Core.EntitySession;
using VSTalk.Model;
using VSTalk.Tools;

namespace VSTalk.Engine.ViewModel.ContactList.ClientResources
{
    class InterlocuterSummaryViewModel:Control
    {
        private IInterlocutorContext _interlocutorContext;

        private Interlocutor _interlocutor;

        private ObservableCollection<InterlocutorResourceItemViewModel> _interlocutorResources;

        public IInterlocutorContext InterlocutorContext
        {
            get
            {
                return _interlocutorContext;
            }
        }

        public ObservableCollection<InterlocutorResourceItemViewModel> InterlocutorResources
        {
            get
            {
                return _interlocutorResources;
            }
        }

        public  InterlocuterSummaryViewModel(Interlocutor interlocutor)
        {
            _interlocutor = interlocutor;
            var resources = SessionModel.GetInterlocutorContext(_interlocutor).Resources;
            _interlocutorResources = new ObservableCollection<InterlocutorResourceItemViewModel>();
            resources.HandleCollectionChanges(initialOrAdd:
                (item) =>
                {
                    
                    var newItem  = new InterlocutorResourceItemViewModel(item);
                    item.SubscribeToChange(() => item.State, sender =>
                    {
                        if (((IInterlocutorResource) sender).State == ContactState.Online)
                        {
                            if (!_interlocutorResources.Any(i => i.ResourceItem == sender))
                                _interlocutorResources.Add(
                                    new InterlocutorResourceItemViewModel((IInterlocutorResource) sender));
                        }
                        else if (_interlocutorResources.Any(i => i.ResourceItem == sender))
                            _interlocutorResources.Remove(
                                _interlocutorResources.FirstOrDefault(i => i.ResourceItem == sender));
                    });
                    if(item.State != ContactState.Offline)
                        _interlocutorResources.Add(newItem);
                },
                removed: (item) => { var toRemove = _interlocutorResources.FirstOrDefault(i => i.ResourceItem == item); if(toRemove != null) 
                                       _interlocutorResources.Remove(toRemove);
                });
            _interlocutorContext = SessionModel.GetInterlocutorContext(_interlocutor);
        }
        
        private static ISessionModel SessionModel
        {
            get { return AppServices.Get<ISessionModel>(); }
        }

    }
}
