using System.ComponentModel;

using VSTalk.Engine.Core.EntitySession;
using VSTalk.Model;

namespace VSTalk.Engine.ViewModel.ContactList.ClientResources
{
    public class InterlocutorResourceViewModel : IClientResource
    {
        private readonly Interlocutor _interlocutor;
        private readonly IInterlocutorContext _context;
        private IResourceUpdateHandler _handler;
        private PropertyChangedEventHandler _eventHandler;

        private static ISessionModel SessionModel
        {
            get { return AppServices.Get<ISessionModel>(); }
        }

        public Interlocutor Interlocutor
        {
            get { return _interlocutor; }
        }

        public InterlocutorResourceViewModel(Interlocutor interlocutor)
        {
            _interlocutor = interlocutor;
            _context = SessionModel.GetInterlocutorContext(interlocutor);
        }


        public void StartListen()
        {
            _eventHandler = (sender, args) =>
            {
                if (_handler != null)
                {
                    _handler.Update();
                }
            };
            _interlocutor.PropertyChanged += _eventHandler;
            _context.PropertyChanged += _eventHandler;

        }

        public void StopListen()
        {
            _interlocutor.PropertyChanged -= _eventHandler;
            _context.PropertyChanged -= _eventHandler;
        }

        public object Model
        {
            get { return _interlocutor; }
        }

        public string Name
        {
            get { return _interlocutor.Name; }
        }

        public int StateOrder
        {
            get
            {
                int showOrder;
                switch (_context.Show)
                {
                    case ShowType.NONE:
                    case ShowType.chat:
                        showOrder = 1;
                        break;
                    case ShowType.away:
                    case ShowType.xa:
                        showOrder = 2;
                        break;
                    case ShowType.dnd:
                        showOrder = 3;
                        break;
                    default:
                        showOrder = 0;
                        break;
                }
                int stateOrder = (int) _context.State;
                return stateOrder * 10 + showOrder;
            }
        }

        public void Subscribe(IResourceUpdateHandler handler)
        {
            _handler = handler;
        }
    }
}