﻿using System;

using VSTalk.Engine.Core.ClientManager;
using VSTalk.Engine.Utils;

namespace VSTalk.Engine.ViewModel.Editors.ClientEditor
{
    public class ClientEditorViewModel
    {
        public ClientInfo Client { get; private set; }

        public RelayCommand SaveChanges { get; set; }

        public RelayCommand RejectChanges { get; set; }

        private static IClientManager ClientManager
        {
            get { return AppServices.Get<IClientManager>(); }
        }

        public event Action Saved;

        public event Action Rejected;

        public ClientEditorViewModel()
        {
            Client = new ClientInfo();
            SetCommands();
        }

        private void SetCommands()
        {
            RejectChanges = new RelayCommand(RejectChangesExecuted);
            SaveChanges = new RelayCommand(SaveChangesExecuted, SaveChangesCanExecute);
        }

        private void RejectChangesExecuted()
        {
            OnRejected();
        }

        private void SaveChangesExecuted()
        {
            ClientManager.AddClient(Client);
            OnSaved();
        }

        private bool SaveChangesCanExecute()
        {
            return !string.IsNullOrEmpty(Client.Login) &&
                   !string.IsNullOrEmpty(Client.Password) &&
                   !string.IsNullOrEmpty(Client.Server);
        }

        protected virtual void OnSaved()
        {
            var handler = Saved;
            if (handler != null) handler();
        }

        protected virtual void OnRejected()
        {
            var handler = Rejected;
            if (handler != null) handler();
        }
    }
}