﻿using System.Windows.Documents;

using VSTalk.Engine.Core.Context;
using VSTalk.Engine.Core.Feed;

using agsXMPP;

using VSTalk.Model;

namespace VSTalk.Engine.ViewModel.Messenger.ThreadRenderer
{
    public class StatusBlockCreator
    {
        private readonly Client _client;

        public StatusBlockCreator(Client client)
        {
            _client = client;
        }

        public IModelContext ModelContext
        {
            get { return AppServices.Get<IModelContext>(); }
        }

        public RenderedElement CreateBlock(IMessage message)
        {
            var status = new Paragraph();
            status.Inlines.Add(CreateStatus(message));
            return new RenderedElement
            {
                    Element = status,
                    Message = message
            };
        }

        private Inline CreateStatus(IMessage message)
        {
            var from = message.XmppMessage.From;
            var name = GetName(from);
            var chatstate = message.XmppMessage.Chatstate.ToString();
            var span = new Span();
            span.Inlines.Add(string.Format("{0} is {1}...", name, chatstate));
            return new Italic(span);
        }

        private string GetName(Jid @from)
        {
            if (@from == null) return "stranger"; // :)
            var client = ModelContext.GetClientByJid(from);
            if (client != null)
            {
                return client.Login;
            }
            var interlocutor = ModelContext.GetContactByJid(_client.Id, @from);
            if (interlocutor != null)
            {
                return interlocutor.Name;
            }
            var room = ModelContext.GetRoomByJid(_client.Id, @from);
            if (room != null)
            {
                return @from.Resource ?? room.Name;
            }
            return from;
        }
    }
}