﻿using VSTalk.Engine.Core.Feed;

using agsXMPP.protocol.extensions.chatstates;

using VSTalk.Model;

namespace VSTalk.Engine.ViewModel.Messenger.ThreadRenderer
{
    public class StatusMessageRenderer : IMessageRenderer
    {
        private readonly EntityThread _thread;

        private readonly StatusBlockCreator _blockCreator;

        private RenderedElement _currentStatus;

        public StatusMessageRenderer(EntityThread thread, Client client)
        {
            _thread = thread;
            _blockCreator = new StatusBlockCreator(client);
        }

        public void Push(IMessage message)
        {
            if (message.XmppMessage.Chatstate == Chatstate.None)
            {
                return;
            }
            if (message.XmppMessage.Chatstate != Chatstate.composing)
            {
                RemoveCurrent();
                return;
            }
            var newStatus = _blockCreator.CreateBlock(message);
            RemoveCurrent();
            _thread.Footer.Blocks.Add(newStatus.Element);
            _currentStatus = newStatus;
        }

        private void RemoveCurrent()
        {
            if (_currentStatus != null)
            {
                if (_thread.Footer.Blocks.Contains(_currentStatus.Element))
                {
                    _thread.Footer.Blocks.Remove(_currentStatus.Element);
                }
            }
        }
    }
}